#include <stdlib.h>
#include <map>
#include <string>
#include <iostream>
#include <sstream>
#include <fstream>
#include <iterator>
#include <math.h>

#include "Fasta_reader.hpp"
#include "sequenceUtil.hpp"
#include "argProcessor.hpp"
#include "DeBruijnGraph.hpp"
#include "string_util.hpp"

using namespace std;







// prototypes
string usage (ArgProcessor args);
int fastaToDeBruijn (int argc, char* argv[]);
void printDeBruijnGraph (string fasta_filename, int kmer_length, int component_val, bool sStrand);





string usage (ArgProcessor args) {
    
    stringstream usage_info;
    
    usage_info 
        << endl << endl
        
        << "**Required" << endl
        << "  --fasta  <str>      " << ":fasta file containing reads" << endl
        << "  -K  <int>           " << ":fasta file containing kmers" << endl
        << "  -C <int>            " << ":file listing filenames containing kmers" << endl
        << endl
        << " **Optional" << endl
        << "  --SS                " << ":indicates strand-specific" << endl << endl;
    
    return(usage_info.str());
    
}



int fastaToDeBruijn (int argc, char* argv[]) {

    try {
        
        ArgProcessor args(argc, argv);
        
        /* Check for essential options */
        if (args.isArgSet("--help") || args.isArgSet("-h")
            ||  (! (args.isArgSet("--fasta") && args.isArgSet("-K") && args.isArgSet("-C")) ) 
            ) {
            
            cerr  << usage(args) << endl << endl;
            
            return 1;
        }
    
        // required
        string fasta_filename = args.getStringVal("--fasta");
        int kmer_length = args.getIntVal("-K");
        int component_val = args.getIntVal("-C");
        
        // optional
        bool sStrand = false;
        if (args.isArgSet("--SS")) {
            sStrand = true;
        }
        
        printDeBruijnGraph(fasta_filename, kmer_length, component_val, sStrand);
        
    }
    
    catch(exception& e) {
        cerr << "error: " << e.what() << "\n";
        return 1;
    }
    
    
    
    return(0);
    
}

void printDeBruijnGraph (string fasta_filename, int kmer_length, int component_val, bool sStrand) {

    Fasta_reader fasta_reader(fasta_filename);
    
    DeBruijnGraph g(kmer_length);

    while (fasta_reader.hasNext()) {
        
        Fasta_entry fe = fasta_reader.getNext();
        string sequence = fe.get_sequence();
        
        vector<string> seq_regions;
        string_util::tokenize(sequence, seq_regions, "X"); // inchworm bundles concatenated with 'X' delimiters by Chrysalis 
        
        for (int s = 0; s < seq_regions.size(); s++) { 
            
            string& seq_region = seq_regions[s];
            
            
            g.add_sequence(seq_region);
            
            if (! sStrand) {
                string revseq = revcomp(seq_region);
                
                g.add_sequence(revseq);
                
            }
        }
    }
    
    //cout << g.toString();
    
    cout << g.toChrysalisFormat(component_val, sStrand);
    


} 



    
int main (int argc, char* argv[]) {
    
    try {
        return(fastaToDeBruijn(argc, argv));
    }
    
    catch (string err) {
        cerr << err << endl;
    }
    
    return(1);
    
}

    
